@@ -89,9 +89,12 @@ class SnowflakeAzureUtil(object):
                 encryption_metadata=None
             )
         except AzureHttpError as err:
-            if (
-                    err.status_code == 403 and "Signature not valid in the specified time frame" in str(
-                    err)):
+            logger.debug(u"Caught exception's status code: {status_code} and message: {ex_representation}".format(
+                status_code=err.status_code,
+                ex_representation=str(err)
+            ))
+            if err.status_code == 403 and ("Signature not valid in the specified time frame" in str(err) or
+                                           "Server failed to authenticate the request." in str(err)):
                 logger.debug(u"AZURE Token expired. Renew and retry")
                 meta[u'result_status'] = ResultStatus.RENEW_TOKEN
                 return None
@@ -183,9 +186,12 @@ class SnowflakeAzureUtil(object):
             meta[u'dst_file_size'] = meta[u'upload_size']
             meta[u'result_status'] = ResultStatus.UPLOADED
         except AzureHttpError as err:
-            if (
-                    err.status_code == 403 and "Signature not valid in the specified time frame" in str(
-                    err)):
+            logger.debug(u"Caught exception's status code: {status_code} and message: {ex_representation}".format(
+                status_code=err.status_code,
+                ex_representation=str(err)
+            ))
+            if err.status_code == 403 and ("Signature not valid in the specified time frame" in str(err) or
+                                           "Server failed to authenticate the request." in str(err)):
                 logger.debug(u"AZURE Token expired. Renew and retry")
                 meta[u'result_status'] = ResultStatus.RENEW_TOKEN
                 return None
@@ -229,9 +235,12 @@ class SnowflakeAzureUtil(object):
 
             meta[u'result_status'] = ResultStatus.DOWNLOADED
         except AzureHttpError as err:
-            if (
-                    err.status_code == 403 and "Signature not valid in the specified time frame" in str(
-                    err)):
+            logger.debug(u"Caught exception's status code: {status_code} and message: {ex_representation}".format(
+                status_code=err.status_code,
+                ex_representation=str(err)
+            ))
+            if (err.status_code == 403 and ("Signature not valid in the specified time frame" in str(err) or
+                                            "Server failed to authenticate the request." in str(err))):
                 logger.debug(u"AZURE Token expired. Renew and retry")
                 meta[u'result_status'] = ResultStatus.RENEW_TOKEN
                 return None
