@@ -10,7 +10,7 @@ import sys
 import uuid
 from logging import getLogger
 from threading import (Timer, Lock)
-
+from base64 import b64decode
 from six import u
 
 from .compat import (BASE_EXCEPTION_CLASS)
@@ -30,6 +30,13 @@ from .file_transfer_agent import (SnowflakeFileTransferAgent)
 from .sqlstate import (SQLSTATE_FEATURE_NOT_SUPPORTED)
 from .telemetry import (TelemetryData, TelemetryField)
 from .time_util import get_time_millis
+from .chunk_downloader import ArrowChunkIterator
+from .converter_arrow import SnowflakeArrowConverter
+
+try:
+    from pyarrow.ipc import open_stream
+except ImportError:
+    pass
 
 STATEMENT_TYPE_ID_DML = 0x3000
 STATEMENT_TYPE_ID_INSERT = STATEMENT_TYPE_ID_DML + 0x100
@@ -460,6 +467,8 @@ class SnowflakeCursor(object):
                 query = command
                 processed_params = self._connection._process_params_qmarks(
                     params, self)
+        except ValueError:
+            raise
         except Exception:
             self.connection.incident.report_incident()
             raise
@@ -568,6 +577,7 @@ class SnowflakeCursor(object):
 
     def chunk_info(self, data, use_ijson=False):
         is_dml = self._is_dml(data)
+        self._query_result_format = data.get(u'queryResultFormat', u'json')
 
         if self._total_rowcount == -1 and not is_dml and data.get(u'total') \
                 is not None:
@@ -576,6 +586,10 @@ class SnowflakeCursor(object):
         self._description = []
         self._column_idx_to_name = {}
         self._column_converter = []
+
+        converter = SnowflakeArrowConverter() if \
+            self._query_result_format == 'arrow' else self._connection.converter
+
         for idx, column in enumerate(data[u'rowtype']):
             self._column_idx_to_name[idx] = column[u'name']
             type_value = FIELD_NAME_TO_ID[column[u'type'].upper()]
@@ -587,15 +601,21 @@ class SnowflakeCursor(object):
                                       column[u'scale'],
                                       column[u'nullable']))
             self._column_converter.append(
-                self._connection.converter.to_python_method(
-                    column[u'type'].upper(), column))
+                    converter.to_python_method(
+                        column[u'type'].upper(), column))
 
         self._total_row_index = -1  # last fetched number of rows
 
         self._chunk_index = 0
         self._chunk_count = 0
-        self._current_chunk_row = iter(data.get(u'rowset'))
-        self._current_chunk_row_count = len(data.get(u'rowset'))
+        if self._query_result_format == 'arrow':
+            # result as arrow chunk
+            arrow_bytes = b64decode(data.get(u'rowsetBase64'))
+            arrow_reader = open_stream(arrow_bytes)
+            self._current_chunk_row = ArrowChunkIterator(arrow_reader)
+        else:
+            self._current_chunk_row = iter(data.get(u'rowset'))
+            self._current_chunk_row_count = len(data.get(u'rowset'))
 
         if u'chunks' in data:
             chunks = data[u'chunks']
@@ -617,6 +637,7 @@ class SnowflakeCursor(object):
             logger.debug(u'qrmk=%s', qrmk)
             self._chunk_downloader = self._connection._chunk_downloader_class(
                 chunks, self._connection, self, qrmk, chunk_headers,
+                query_result_format=self._query_result_format,
                 prefetch_threads=self._connection.client_prefetch_threads,
                 use_ijson=use_ijson)
 
